import 'dart:async';

import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/foundation.dart';
import 'package:hive_flutter/hive_flutter.dart';

abstract class IStorage<T> {
  Future<void> openBox(
    String boxName, {
    bool useEncryption = true,
  });
  Future<void> init({
    String? subdir,
  });
  void registerAdapter<S>(
    TypeAdapter<S> adapter, {
    bool internal = false,
    bool override = false,
  });
  void registerAdapterManual({required Function(HiveInterface) adapter});
  Future<void> openLazyBox(
    String boxName, {
    bool useEncryption = true,
  });
  bool isOpen();
  Future<void> putData({
    required Map<dynamic, T> data,
    bool autoClose = false,
  });
  Future<void> putDatum({
    required String key,
    required T value,
    bool autoClose = false,
  });
  Future<void> putListData({
    required Iterable<T> dataList,
    bool autoClose = false,
  });

  FutureOr<T?> getData({
    required String key,
    bool autoClose = false,
  });
  Future<IList<T>?> getListData({
    bool autoClose = false,
  });
  // Future<T?> getLazyData({
  //   required String key,
  // });
  Future<IList<T>?> getLazyListData({
    bool autoClose = false,
  });
  Future<void> clear();
  Future<void> delete({
    required String key,
    bool autoClose = false,
  });
  Future<void> deleteList({
    required Iterable<dynamic> key,
    bool autoClose = false,
  });
  ValueListenable<Box<T>> listenable({
    List? keys,
  });
  ValueListenable<LazyBox<T>> lazyListenable({
    List? keys,
  });

  Future<dynamic> close();
}
